//=====================================================================================================



// Copyright ?2014 Point Grey Research, Inc. All Rights Reserved.
//

// This software is the confidential and proprietary information of Point
// Grey Research, Inc. ("Confidential Information").  You shall not
// disclose such Confidential Information and shall use it only in
// accordance with the terms of the license agreement you entered into
// with PGR.

//
// PGR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
// SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE, OR NON-INFRINGEMENT. PGR SHALL NOT BE LIABLE FOR ANY DAMAGES
// SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
// THIS SOFTWARE OR ITS DERIVATIVES.

//=====================================================================================================
//=====================================================================================================
//
// This example will benchmark your Point Grey camera for the following 
//    1. dropped frames
//    2. average grab time between images
//    3. average frame rate
//    4. minimum grab time between images
//    5. maximum grab time between images
//
//  In addition, this example will save all the grab time between images into an offline file
//=====================================================================================================

#include "stdafx.h"
#include <Windows.h>
#include "FlyCapture2.h"
#include <process.h>
#include <vector>
#include <fstream>
#include <iterator>
#include <iostream>
#include <algorithm>
#include <time.h>
#include <ctime>
//#include <afxtempl.h>
using namespace FlyCapture2;


// Global Objects


Error error;

//busMgr 
BusManager busMgr;

unsigned int totalNumImages = 0;
float framerateBeSet = NULL;

namespace PtgreyTimer
{
	namespace sys
	{
		///<summary>High precision timer.</summary>
		class Timer
		{
		public:
			struct Query
			{
				inline static unsigned long long GetFrequencyTicks()
				{
					LARGE_INTEGER freq;
					QueryPerformanceFrequency(&freq);
					return static_cast<long long&>(freq.QuadPart);
				}

				inline static unsigned long long GetTicks()
				{
					LARGE_INTEGER ticks;
					QueryPerformanceCounter(&ticks);
					return static_cast<long long&>(ticks.QuadPart);
				}

			private:
				Query(); // cannot create instances; static only
			};


			Timer() : m_ticksStart(Query::GetTicks()) {}

			inline double GetSeconds() const
			{

				const unsigned long long ticksNow = Query::GetTicks();
				const unsigned long long ticksElapsed = ticksNow - m_ticksStart;
				return static_cast<double>(ticksElapsed) / s_frequencyDoublePrecision();
			}

			inline void Reset() { m_ticksStart = Query::GetTicks(); }


		private:
			inline static const unsigned long long& s_frequencyRaw()
			{

				static const unsigned long long s_frequency_data = Query::GetFrequencyTicks();
				return s_frequency_data;
			}

			inline static const double& s_frequencyDoublePrecision()
			{

				static const double s_frequency_data = static_cast<double>(Query::GetFrequencyTicks());
				return s_frequency_data;
			}

			unsigned long long m_ticksStart;

		};
	}

	using namespace sys;
}



inline unsigned int GetSecondCount(unsigned int timestamp)

{

	return ((timestamp & 0xFE000000) >> 25);

	//second_count = (timestamp & 0xFE000000) >> 25 ;

}

inline unsigned int GetCycleCount(unsigned int timestamp)

{

	return  ((timestamp & 0x01FFF000) >> 12);

	//cycle_count =   (timestamp & 0x01FFF000) >> 12 

}


void PrintBuildInfo()
{
	FC2Version fc2Version;
	Utilities::GetLibraryVersion(&fc2Version);
	char version[128];

	sprintf(
		version,
		"FlyCapture2 library version: %d.%d.%d.%d\n",
		fc2Version.major, fc2Version.minor, fc2Version.type, fc2Version.build);

	printf("%s", version);

	char timeStamp[512];
	sprintf(timeStamp, "Application build date: %s %s\n\n", __DATE__, __TIME__);

	printf("%s", timeStamp);
}


void PrintCameraInfo(CameraInfo* pCamInfo)
{
	printf(
		"\n*** CAMERA INFORMATION ***\n"
		"Serial number - %u\n"
		"Camera model - %s\n"
		"Camera vendor - %s\n"
		"Sensor - %s\n"
		"Resolution - %s\n"
		"Firmware version - %s\n"
		"Firmware build time - %s\n\n",
		pCamInfo->serialNumber,
		pCamInfo->modelName,
		pCamInfo->vendorName,
		pCamInfo->sensorInfo,
		pCamInfo->sensorResolution,
		pCamInfo->firmwareVersion,
		pCamInfo->firmwareBuildTime);
}


void PrintError(Error error)
{
	error.PrintErrorTrace();
}

// Image Acquisition Function
int flyCapAcquiFunction(void* inputCamera)
{


	Camera *camera = (Camera*)inputCamera;
	int iCountMissedIm = NULL;
	int iCountMissedIm_byTimestamp = 0;
	int iFrameNumberPrev = 0;
	int iFrameNumberDelta = 0;

	unsigned int iTimestamp = 0;
	unsigned int iTimestampPrev = 0;
	int iTimestampDelta = 0;
	int iTimestampDelta_estimate_from_framerate = 0;
	FILE* pFile = NULL;



	// Set current thread to priority highest
	if (!SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_HIGHEST))
	{
		return 1;
	}

	DWORD dwThreadPri = GetThreadPriority(GetCurrentThread());

	if (dwThreadPri == 2)
	{
		printf("\nAcquisition thread priority is set to Highest\n");
	}

	Image image, firstImage, lastImage;
	ImageMetadata metadata;
	double totalGrabTime = 0;
	double grabTimeS = 0;
	double minTime = 1;
	double maxTime = 0;

#undef min
#undef max
	std::vector<double> GrabTimeProfile;
	//std::vector<double> DroppedFrameProfile;




	// Set up the diagnostics info
	const unsigned int k_frameSkippedReg = 0x12C0;
	const unsigned int k_frameTransmittedReg = 0x121C;
	unsigned int frameSkippedRegVal = 0;
	unsigned int frameTransmittedRegVal = 0;


	unsigned int m_previousSkippedImageCount = 0;
	unsigned int m_startingFrameTransmittedCount = 0;
	unsigned int numberOfFramesDroppedFromRegister;
	clock_t t;
	t = clock();

	//Check the starting value of the frameTransmittedRegVal register
	//error = camera.ReadRegister( k_frameTransmittedReg, &m_startingFrameTransmittedCount );
	//std::cout << "\nThe frame transmit register starting count is: " << std::hex << m_startingFrameTransmittedCount << '\n';

	printf("\nGrabbing %d images... \n", totalNumImages);


	/*~~ Add by  ~~*/

	// Estimate the delta timestamp by framerateBeSet

	iTimestampDelta_estimate_from_framerate = (int)(1.0 * 8000.0 / framerateBeSet);  // unit: cycle count, because this is the most accurary of timestamp
																					 //This means in framerante interval time, there are how many cyclecounts , this is a init time

																					 // Open a log file to save some information


	CameraInfo camInfo;
	error = camera->GetCameraInfo(&camInfo);

	char buf[256];
	sprintf(buf, "%d.txt", camInfo.serialNumber);
	pFile = fopen(buf, "w+");
	if (pFile != NULL)
	{
		printf("\nOpen file to save log...");
		fprintf(pFile, "\nFrame rate is: %f\nEstimate delta timestamp: %d\n\n", framerateBeSet, iTimestampDelta_estimate_from_framerate);
		fprintf(pFile, "current local system time\t Embedded frame counter\t Embedded timestamp\t Delta frame counter\t Delta timestamp(unit: cycle count)\t Framecounter missed\t Framecounter missed(fromtimestamp)\n ");

	}



	time_t rawtime;
	struct tm* timeinfo;
	char buffer[80];

	time(&rawtime);
	timeinfo = localtime(&rawtime);

	// Start benchmark ...
	for (unsigned int i = 0; i < totalNumImages; i++)
	{
		// Grab image 
		PtgreyTimer::Timer grabTimer;

		// check the embedded transmitted FC value on the first image grabbed
		if (i == 0)
		{
			error = camera->RetrieveBuffer(&firstImage);
			metadata = firstImage.GetMetadata();
		}

		// check the embedded transmitted FC value on the last image grabbed
		else if (i == totalNumImages - 1)
		{
			error = camera->RetrieveBuffer(&lastImage);
			metadata = lastImage.GetMetadata();
		}
		else
		{
			error = camera->RetrieveBuffer(&image);
			metadata = image.GetMetadata();
		}
		if (error != PGRERROR_OK)
		{
			PrintError(error);
			//return -1;
		}

		grabTimeS = grabTimer.GetSeconds();
		grabTimer.Reset();
		totalGrabTime = totalGrabTime + grabTimeS;


		// Profile grab time
		GrabTimeProfile.push_back(grabTimeS);

		// Find minimum and maximum grab time
		minTime = std::min(minTime, grabTimeS);
		maxTime = std::max(maxTime, grabTimeS);

		if (error != PGRERROR_OK)
		{
			PrintError(error);
			break;
		}


		//Check to see if there is any skipped frames using skip frame register 0x12C0
		error = camera->ReadRegister(k_frameSkippedReg, &frameSkippedRegVal);
		const unsigned int skippedFrames = frameSkippedRegVal & 0x7FFFFFFF;
		if (i == 0)
		{
			m_previousSkippedImageCount = skippedFrames;
		}
		else if (skippedFrames != m_previousSkippedImageCount)
		{
			// Skipped a frame
			/*DroppedFrameProfile.push_back(1);*/
			numberOfFramesDroppedFromRegister += skippedFrames  - m_previousSkippedImageCount;		
			m_previousSkippedImageCount = skippedFrames;

		}


		// Keep track of the difference in image sequence numbers (iFrameCount)
		// in order to determine if any images have been missed.  A difference
		// greater than 1 indicates that an image has been missed.
		if (i == 0)
		{
			// This is the first image, set up the variables
			iFrameNumberPrev = metadata.embeddedFrameCounter;
			iFrameNumberDelta = 1;

			iTimestamp = metadata.embeddedTimeStamp; // Add by Giles
			iTimestampDelta = iTimestampDelta_estimate_from_framerate; // Add by Giles
		}
		else
		{
			// Get the difference in sequence numbers between the current
			// image and the last image we received
			iFrameNumberDelta = metadata.embeddedFrameCounter - iFrameNumberPrev;
			iTimestamp = metadata.embeddedTimeStamp; // Add by Giles			

													 /*~~ Add by ~~*/
			if (GetSecondCount(iTimestamp) < GetSecondCount(iTimestampPrev))
			{
				iTimestampDelta = (GetSecondCount(iTimestamp) + 128 - GetSecondCount(iTimestampPrev)) * 8000 + GetCycleCount(iTimestamp) - GetCycleCount(iTimestampPrev);

			}
			else
				iTimestampDelta = (GetSecondCount(iTimestamp) - GetSecondCount(iTimestampPrev)) * 8000 + GetCycleCount(iTimestamp) - GetCycleCount(iTimestampPrev);

			// unit of iTimestampDelta: cycle count's unit (1/8000 second)

			/*~~ Add by ~~*/
		}
		if (iFrameNumberDelta != 1)
		{
			iCountMissedIm += iFrameNumberDelta - 1;
		}

		/*~~ Add by  ~~*/

		if (abs(iTimestampDelta - iTimestampDelta_estimate_from_framerate) > 2)
			// If delta timestamp has difference more than 2 with timestamp calculated by framerate
		{
			//iCountMissedIm_byTimestamp += 1;
			iCountMissedIm_byTimestamp += (((float)iTimestampDelta / iTimestampDelta_estimate_from_framerate) + 0.5) - 1;
		}

		/*~~ Add by Giles ~~*/
		iFrameNumberPrev = metadata.embeddedFrameCounter;
		iTimestampPrev = iTimestamp; // Add by Giles


		strftime(buffer, 80, "(UTC)%X %x", timeinfo);

		if (pFile != NULL)
		{

			fprintf(pFile, "%10s\t %5d\t %X \t%5d\t %5d\t %5d\t %5d %5d\n", buffer, iFrameNumberPrev, iTimestampPrev, iFrameNumberDelta, iTimestampDelta, iCountMissedIm, iCountMissedIm_byTimestamp, numberOfFramesDroppedFromRegister);


			// write some information to the file


		}
	}

	float fps = 0;
	t = clock() - t;
	float time = (float)t / CLOCKS_PER_SEC;
	fps = totalNumImages / time;
	printf("\nThe whole Grab loop used %f s \n", time);
	printf("\nUsing the whole grab loop time, the speed the camera grabbed on average was %f fps \n", fps);

	//Check the ending value of the frameTransmittedRegVal register
	//error = camera.ReadRegister( k_frameTransmittedReg, &frameTransmittedRegVal );
	//std::cout << "\nThe frame transmit register ending count is: " << std::hex << frameTransmittedRegVal << '\n';
	//std::cout << "\nThe frame transmit from the camera during this test (based on the transmit FC register change during the test) is: " << std::dec << (frameTransmittedRegVal - m_startingFrameTransmittedCount) << '\n';
	char firstImageName[256];
	char lastImageName[256];
	sprintf(firstImageName, "firstImage%d.raw", camInfo.serialNumber);
	sprintf(lastImageName, "lastImage%d.raw", camInfo.serialNumber);

	//const char* firstImageName = "firstImage.raw";
	//const char* lastImageName = "lastImage.raw";

	error = firstImage.Save(firstImageName, ImageFileFormat::RAW);
	if (error != PGRERROR_OK)
	{
		PrintError(error);
		//return -1;
	}

	error = lastImage.Save(lastImageName, ImageFileFormat::RAW);
	if (error != PGRERROR_OK)
	{
		PrintError(error);
		//return -1;
	}

	unsigned char* firstImageData = firstImage.GetData();
	unsigned char* lastImageData = lastImage.GetData();

	unsigned int lastImageEmbeddedTimestamp = unsigned int(lastImageData[0] << 24) + unsigned int(lastImageData[1] << 16) + unsigned int(lastImageData[2] << 8) + unsigned int(lastImageData[3]);

	unsigned int lastImageEmbeddedFrameCounter = unsigned int(lastImageData[4] << 24) + unsigned int(lastImageData[5] << 16) + unsigned int(lastImageData[6] << 8) + unsigned int(lastImageData[7]);

	unsigned int lastImageEmbeddedTransmittedFC = unsigned int(lastImageData[8] << 24) + unsigned int(lastImageData[9] << 16) + unsigned int(lastImageData[10] << 8) + unsigned int(lastImageData[11]);



	unsigned int firstImageEmbeddedTimestamp = unsigned int(firstImageData[0] << 24) + unsigned int(firstImageData[1] << 16) + unsigned int(firstImageData[2] << 8) + unsigned int(firstImageData[3]);

	unsigned int firstImageEmbeddedFrameCounter = unsigned int(firstImageData[4] << 24) + unsigned int(firstImageData[5] << 16) + unsigned int(firstImageData[6] << 8) + unsigned int(firstImageData[7]);

	unsigned int firstImageEmbeddedTransmittedFC = unsigned int(firstImageData[8] << 24) + unsigned int(firstImageData[9] << 16) + unsigned int(firstImageData[10] << 8) + unsigned int(firstImageData[11]);



	unsigned int embeddedTransmittedFCdifference = lastImageEmbeddedTransmittedFC - firstImageEmbeddedTransmittedFC;

	unsigned int ImageTransmitted = 1 + embeddedTransmittedFCdifference;


	fprintf(pFile, "\nfistImage's first embeddedImageInfo: %u, second: %u, third: %u\n", firstImageEmbeddedTimestamp, firstImageEmbeddedFrameCounter, firstImageEmbeddedTransmittedFC);

	metadata = firstImage.GetMetadata();

	fprintf(pFile, "\nfirstImage's meta data's embeddedTimestamp: %u, embeddedFrameCounter: %u\n", metadata.embeddedTimeStamp, metadata.embeddedFrameCounter);



	fprintf(pFile, "\nlastImage's first embeddedImageInfo: %u, second: %u, third: %u\n", lastImageEmbeddedTimestamp, lastImageEmbeddedFrameCounter, lastImageEmbeddedTransmittedFC);

	metadata = lastImage.GetMetadata();

	fprintf(pFile, "\nlastImage's meta data's embeddedTimestamp: %u, embeddedFrameCounter: %u\n", metadata.embeddedTimeStamp, metadata.embeddedFrameCounter);

	// close the log file

	if (pFile != NULL)

		fclose(pFile); // Add by Giles



	std::cout << "\nNumber of image transmitted off of the camera (based on embedded trasmit FC register in the first/last images) is: " << ImageTransmitted << '\n';


	// Dropped Frame Stat
	printf("\nDropped frames based on skipped frames register on camera is: %d \n", numberOfFramesDroppedFromRegister);
	printf("\nDropped frames based on the continuity of embedded framecounter on each images: %d \n", iCountMissedIm);
	printf("\nDropped frames based on the constant delta embedded timestamp on each images: %d \n", iCountMissedIm_byTimestamp); // Add by Giles



																																 // Average grab time
	printf("\nAverage grab time for %d images is %f ms \n", totalNumImages, (totalGrabTime / totalNumImages) * 1000);


	// Average frame rate
	printf("\nAverage frame rate based on RetrieveBuffer() for %d images is %f fps \n", totalNumImages, 1 / ((totalGrabTime / totalNumImages)));


	//Save grabTime to file
	std::ofstream outFile("./grabTime.txt");
	std::ostream_iterator<double> output_iterator(outFile, "\n");
	std::copy(GrabTimeProfile.begin(), GrabTimeProfile.end(), output_iterator);

	printf("\nMinimum grab time for %d images is %f ms \n", totalNumImages, minTime * 1000);
	printf("\nMaximum grab time for %d images is %f ms \n", totalNumImages, maxTime * 1000);

	return 0;
}

void display_Usage(const char* pszProgramName)
{
	printf("\n\n%s Usage: Input number of images to capture:", pszProgramName);
}

void processArguments(int argc, char* argv[])
{
	const char* pszProgname = argv[0];
	int   iOpt;
	char* pszCurrParam;
	bool  bBadArgs = false;

	// no arguments?
	if (argc == 1)
	{
		display_Usage(pszProgname);
		/*printf("\n<PRESS ANY KEY TO EXIT>\n");
		getchar();
		exit( 0);*/
	}

	totalNumImages = atoi(argv[1]);
	if (bBadArgs)
	{
		display_Usage(pszProgname);
		exit(0);
	}
}
//camera init and start acquiring
unsigned int __stdcall initAcquire(void * pGuid)
{

	PGRGuid * guid = (PGRGuid *)pGuid;
	// Get the GUID for the first connected camera
	//	error = busMgr.GetCameraFromIndex(0, guid);
	Camera cam;

	// Connect to a camera
	error = cam.Connect(guid);
	if (error != PGRERROR_OK)
	{
		PrintError(error);
		return -1;
	}

	// Power on the camera
	const unsigned int k_cameraPower = 0x610;
	const unsigned int k_powerVal = 0x80000000;
	error = cam.WriteRegister(k_cameraPower, k_powerVal);
	if (error != PGRERROR_OK)
	{
		PrintError(error);
		return -1;
	}

	const unsigned int millisecondsToSleep = 100;
	unsigned int regVal = 0;

	// Wait for camera to complete power-up
	do
	{
#if defined(WIN32) || defined(WIN64)
		Sleep(millisecondsToSleep);
#else
		usleep(millisecondsToSleep * 1000);
#endif

		error = cam.ReadRegister(k_cameraPower, &regVal);
		if (error != PGRERROR_OK)
		{
			PrintError(error);
			return -1;
		}
	} while ((regVal & k_powerVal) == 0);


	// Get the camera information
	CameraInfo camInfo;
	error = cam.GetCameraInfo(&camInfo);
	if (error != PGRERROR_OK)
	{
		PrintError(error);
		return -1;
	}

	PrintCameraInfo(&camInfo);

	// Get current trigger settings
	TriggerMode triggerMode;
	error = cam.GetTriggerMode(&triggerMode);
	if (error != PGRERROR_OK)
	{
		PrintError(error);
		return -1;
	}

	// Set camera to trigger mode to off
	triggerMode.onOff = false;
	triggerMode.mode = 0;
	triggerMode.parameter = 0;
	triggerMode.source = 0;

	error = cam.SetTriggerMode(&triggerMode);
	if (error != PGRERROR_OK)
	{
		PrintError(error);
		return -1;
	}

	// Get the camera configuration
	FC2Config config;
	error = cam.GetConfiguration(&config);
	if (error != PGRERROR_OK)
	{
		PrintError(error);
		return -1;
	}

	// Set camera to use maximum frame rate
	PropertyInfo fpsPropInfo;
	fpsPropInfo.type = FRAME_RATE;
	error = cam.GetPropertyInfo(&fpsPropInfo);
	if (error != PGRERROR_OK)
	{
		PrintError(error);
		return -1;
	}

	Property fpsProp;
	fpsProp.type = FRAME_RATE;
	error = cam.GetProperty(&fpsProp);
	if (error != PGRERROR_OK)
	{
		PrintError(error);
		return -1;
	}

	fpsProp.onOff = true;
	fpsProp.autoManualMode = false;
	fpsProp.absControl = true;
	fpsProp.absValue = fpsPropInfo.absMax;
    framerateBeSet = fpsPropInfo.absMax;

	error = cam.SetProperty(&fpsProp);
	if (error != PGRERROR_OK)
	{
		PrintError(error);
		return -1;
	}

	// Set the grab time out to 5 seconds
	config.grabTimeout = 5000;
	// Set grab mode to buffer frames (by default buffer frames mode will allocate 10 buffers)
	config.grabMode = BUFFER_FRAMES;
	// Allocate more buffers just in case..
	config.numBuffers = 250;

	// Enable high performance mode
	config.highPerformanceRetrieveBuffer = true;
	// Set the camera configuration
	error = cam.SetConfiguration(&config);
	if (error != PGRERROR_OK)
	{
		PrintError(error);
		return -1;
	}

	Property prop;
	prop.type = SHUTTER;
	error = cam.GetProperty(&prop);
	if (error != PGRERROR_OK)
	{
		PrintError(error);
		return -1;
	}

	// Set shutter to 0.5 ms
	prop.absControl = true;
	prop.absValue = 0.5;
	prop.autoManualMode = false;
	prop.onOff = true;

	error = cam.SetProperty(&prop);
	if (error != PGRERROR_OK)
	{
		PrintError(error);
		return -1;
	}



	EmbeddedImageInfo m_embeddedInfo;
	//*********************Embedded shutter**********************
	error = cam.GetEmbeddedImageInfo(&m_embeddedInfo);
	if (error != PGRERROR_OK)
	{
		PrintError(error);
		return -1;
	}

	m_embeddedInfo.timestamp.onOff = true;
	//m_embeddedInfo.shutter.onOff = true;
	m_embeddedInfo.frameCounter.onOff = true;

	error = cam.SetEmbeddedImageInfo(&m_embeddedInfo);
	if (error != PGRERROR_OK)
	{
		PrintError(error);
		return -1;
	}


	// turn on embedded image transmit frame counter
	unsigned int FrameInfoRegValue;
	cam.ReadRegister(0x12F8, &FrameInfoRegValue);
	if (error != PGRERROR_OK)
	{
		PrintError(error);
		return -1;
	}

	FrameInfoRegValue = FrameInfoRegValue | 0x400;
	cam.WriteRegister(0x12F8, FrameInfoRegValue);
	if (error != PGRERROR_OK)
	{
		PrintError(error);
		return -1;
	}

	//**********************************************
	// Camera is ready, start capturing images
	error = cam.StartCapture();
	if (error != PGRERROR_OK)
	{
		PrintError(error);
		return -1;
	}

	printf("Started Image Capture.\n");

	flyCapAcquiFunction(&cam);
#pragma region cleanup
	// Turn trigger mode off.
	triggerMode.onOff = false;
	error = cam.SetTriggerMode(&triggerMode);
	if (error != PGRERROR_OK)
	{
		PrintError(error);
		return -1;
	}

	printf("\nFinished grabbing images\n");

	// Stop capturing images
	error = cam.StopCapture();
	if (error != PGRERROR_OK)
	{
		PrintError(error);
		return -1;
	}


	// Disconnect the camera
	error = cam.Disconnect();
	if (error != PGRERROR_OK)
	{
		PrintError(error);
		return -1;
	}

	return 0;
}

struct ThreadInfo
{
	unsigned int m_nID;
	HANDLE m_pHandle;
};
#include <iostream> 
#include <list> 
#include <numeric> 
#include <algorithm> 

using namespace std;

int main(int argc, char* argv[])
{
	processArguments(argc, argv);

	// Set process priority class to high
	if (!SetPriorityClass(GetCurrentProcess(), HIGH_PRIORITY_CLASS))
	{
		//Fail to set Priority
		return -1;
	}

	DWORD dwPriClass = GetPriorityClass(GetCurrentProcess());
	if (dwPriClass == 0x00000080)
	{
		printf("\nCurrent process priority class is HIGH_PRIORITY_CLASS\n");
	}

	// Set Processor Affinity to 1 (i.e. only use CPU 0)

	HANDLE process = GetCurrentProcess();
	DWORD_PTR processAffinityMask;
	DWORD_PTR systemAffinityMask;

	if (!GetProcessAffinityMask(process, &processAffinityMask, &systemAffinityMask))
	{
		return -1;
	}
	
	// Use core #1
	const int core = 1;

	//// Use one core via masking
	//dword_ptr mask=0x1;
	//for (int bit=0, currentcore=1; bit < 64; bit++)
	//{
	//if (mask & processaffinitymask)
	//{
	//if (currentcore != core)
	//{
	//processaffinitymask &= ~mask;
	//}
	//currentcore++;
	//}
	//mask = mask << 1;
	//}

	//bool success = setprocessaffinitymask(process, processaffinitymask);
	//if (!success)
	//{
	//return -1;
	//}
	
	PrintBuildInfo();
	// Enumerate cameras

	unsigned int numCameras;
	error = busMgr.GetNumOfCameras(&numCameras);
	if (error != PGRERROR_OK)
	{
		PrintError(error);
		return -1;
	}
	printf("Number of cameras detected: %u\n", numCameras);

	if (numCameras < 1)
	{
		printf("Insufficient number of cameras... exiting\n");
		return -1;
	}

	PGRGuid guid;
	list<ThreadInfo*>liThread;//Thread Information list
	list<PGRGuid*>liGuid;// Guid list
	for (int i = 0; i<numCameras; i++)
	{

		PGRGuid * guid = new PGRGuid();
		error = busMgr.GetCameraFromIndex(i, guid);
		liGuid.push_back(guid);
		if (error != PGRERROR_OK)
		{
			PrintError(error);
			return -1;
		}
		ThreadInfo* threadInfo = new ThreadInfo();


		// Spawn image acquisition thread
		threadInfo->m_pHandle = reinterpret_cast<HANDLE>(_beginthreadex(0, 0, initAcquire, guid, 0, &(threadInfo->m_nID)));
		liThread.push_back(threadInfo);
	}/**/

	list<ThreadInfo*>::iterator it;
	// wait for all thread finishing.
	while (liThread.size() != 0)
	{
		ThreadInfo* threadInfo = liThread.front();
		WaitForSingleObject(threadInfo->m_pHandle, INFINITE);
		liThread.pop_front();
		delete threadInfo;
	}
	//delete all Guid.
	while (liGuid.size() != 0)
	{
		PGRGuid * guid = liGuid.front();
		delete guid;
		liGuid.pop_front();
	}
#pragma endregion

	printf("Done! Press Enter to exit...\n");
	getchar();

	return 0;
}






