#ifndef CAMERA_CONFIG_H__
#define CAMERA_CONFIG_H__

#include <stdint.h>

#ifdef FLYCAPTURE2_SDK_1404
#include "FlyCapture2Defs_C_1404.h"
#elif defined(FLYCAPTURE2_SDK_1604)
#include "FlyCapture2Defs_C_1604.h"
#else
#error "No proper flycapture SDK defined"
#endif

#include "ret_code.h"

/** @brief Camera configuration types.
 *
 *  @details The configuration types correspond to the FlyCapture2 SDK configuration calls.
 */
typedef enum
{
    CAMERA_CONFIG_fc2GetConfiguration               = 0x01,
    CAMERA_CONFIG_fc2SetConfiguration               = 0x02,
    CAMERA_CONFIG_fc2GetCameraInfo                  = 0x03,
    CAMERA_CONFIG_fc2GetPropertyInfo                = 0x04,
    CAMERA_CONFIG_fc2GetProperty                    = 0x05,
    CAMERA_CONFIG_fc2SetProperty                    = 0x06,
    CAMERA_CONFIG_fc2GetVideoModeAndFrameRateInfo   = 0x07,
    CAMERA_CONFIG_fc2GetVideoModeAndFrameRate       = 0x08,
    CAMERA_CONFIG_fc2SetVideoModeAndFrameRate       = 0x09,
    CAMERA_CONFIG_fc2GetFormat7Info                 = 0x0A,
    CAMERA_CONFIG_fc2ValidateFormat7Settings        = 0x0B,
    CAMERA_CONFIG_fc2GetFormat7Configuration        = 0x0C,
    CAMERA_CONFIG_fc2SetFormat7ConfigurationPacket  = 0x0D,
    CAMERA_CONFIG_fc2SetFormat7Configuration        = 0x0E,
    CAMERA_CONFIG_fc2GetEmbeddedImageInfo           = 0x0F,
    CAMERA_CONFIG_fc2SetEmbeddedImageInfo           = 0x10,
    CAMERA_CONFIG_fc2GetTriggerModeInfo             = 0x11,
    CAMERA_CONFIG_fc2GetTriggerMode                 = 0x12,
    CAMERA_CONFIG_fc2SetTriggerMode                 = 0x13,
    CAMERA_CONFIG_fc2GetTriggerDelayInfo            = 0x14,
    CAMERA_CONFIG_fc2GetTriggerDelay                = 0x15,
    CAMERA_CONFIG_fc2SetTriggerDelay                = 0x16,

    CAMERA_CONFIG_FORCE32_BIT                       = 0xFFFFFFFF
} camera_config_type_t;


/** @brief Configuration data for the FlyCapture2 SDK calls. */
typedef struct
{
    camera_config_type_t type;                          /**< Type of configuration. */

    union
    {
        fc2Config           config;                         /**< Config struct used when type is @ref CAMERA_CONFIG_fc2GetConfiguration or @ref CAMERA_CONFIG_fc2SetConfiguration. */
        fc2CameraInfo       cameraInfo;                     /**< Config struct used when type is @ref CAMERA_CONFIG_fc2GetCameraInfo. */
        fc2PropertyInfo     propertyInfo;                   /**< Config struct used when type is @ref CAMERA_CONFIG_fc2GetPropertyInfo. */
        fc2Property         property;                       /**< Config struct used when type is @ref CAMERA_CONFIG_fc2GetProperty or @ref CAMERA_CONFIG_fc2SetProperty. */
        fc2TriggerModeInfo  triggerModeInfo;                /**< Config struct used when type is @ref CAMERA_CONFIG_fc2GetTriggerModeInfo. */
        fc2TriggerMode      triggerMode;                    /**< Config struct used when type is @ref CAMERA_CONFIG_fc2GetTriggerMode or @ref CAMERA_CONFIG_fc2SetTriggerMode. */
        fc2TriggerDelayInfo triggerDelayInfo;               /**< Config struct used when type is @ref CAMERA_CONFIG_fc2GetTriggerDelayInfo. */
        fc2TriggerDelay     triggerDelay;                   /**< Config struct used when type is @ref CAMERA_CONFIG_fc2GetTriggerDelay or @ref CAMERA_CONFIG_fc2SetTriggerDelay. */

        struct
        {
            fc2VideoMode videoMode;
            fc2FrameRate frameRate;
            BOOL         supported;
        } video_framerate_info;                         /**< Config struct used when type is @ref CAMERA_CONFIG_fc2GetVideoModeAndFrameRateInfo, @ref CAMERA_CONFIG_fc2GetVideoModeAndFrameRate, or @ref CAMERA_CONFIG_fc2SetVideoModeAndFrameRate. */

        struct
        {
            fc2Format7Info  format7Info;
            BOOL            supported;
        } format7Info;                                  /**< Config struct used when type is @ref CAMERA_CONFIG_fc2GetFormat7Info. */

        struct
        {
            fc2Format7ImageSettings imageSettings;
            fc2Format7PacketInfo    packetInfo;
            BOOL                    supported;
        } format7Settings;                              /**< Config struct used when type is @ref CAMERA_CONFIG_fc2ValidateFormat7Settings.*/

        struct
        {
            fc2Format7ImageSettings imageSettings;
            union
			{
            	unsigned int            packetSize;
            	float                   percentage;
            };
        } format7Configuration;                         /**< Config struct used when type is @ref CAMERA_CONFIG_fc2GetFormat7Configuration, CAMERA_CONFIG_fc2SetFormat7ConfigurationPacket, or CAMERA_CONFIG_fc2SetFormat7Configuration.*/

        fc2EmbeddedImageInfo    embeddedImageInfo;
    } data;

} camera_config_t;


/** @brief Sends all camera configuration parameters.
 *
 *	@details All parameters are set in the implementation file.
 *
 *  @return RET_SUCCESS		If configuration was successful
 *  else return some error code.
 */
ret_code_t camera_config_send_all(void);


/** @brief Sends one camera configuration call.
 *
 *  @param[in] p_config		The camera configuration structure.
 *  @param[in] config_size	The size of the configuration parameters.
 *
 *  @return RET_SUCCESS		If configuration was successful
 *  else return some error code.
 */
ret_code_t camera_config(camera_config_t * p_config, size_t config_size);


/** @brief Retrieves the remote camera configuration and prints it.
 */
void print_config_data(void);


/** @brief Retrieves the maximum camera for a given configuration.
 *
 *  @note The retrieved maximum frame rate is not always equal to the actual maximum frame rate.
 */
float camera_check_max_frame_rate(uint32_t width, uint32_t height, fc2PixelFormat pixel_format);


#endif // CAMERA_CONFIG_H__

