#include <pthread.h>
#include <stdio.h>
#include <string.h>

#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h>

#include "log.h"
#include "tcp_client.h"
#include "app_error.h"


int m_client_fd;
static struct sockaddr_in server;
static uint8_t recv_buffer[RECV_BUFFER_SIZE];

#define HEADER_SIZE         5

static ret_code_t try_read_length(uint32_t length)
{
    uint32_t bytes_read, temp_bytes_read;

    bytes_read = 0;
    while ( bytes_read < length)
    {
        temp_bytes_read = recv(m_client_fd,&recv_buffer[bytes_read],length - bytes_read, 0);

        if (temp_bytes_read <= 0)
        {
            return RET_ERROR_OTHER;
        }

        bytes_read += temp_bytes_read;

    }
    return RET_SUCCESS;
}


static void *listener_func(void* args)
{
    ret_code_t          ret_code;
    tcp_evt_t           evt;
    tcp_evt_handler_t   evt_handler;

    evt_handler = (tcp_evt_handler_t)args;

    while (1)
    {
        // read header
        ret_code = try_read_length(HEADER_SIZE);
        if (ret_code != RET_SUCCESS)
        {
            // disconnect
            break;
        }

        evt.type = TCP_EVT_TYPE_RECV_MESSAGE;

        // store header information in the event
        evt.message.type = (message_type_t)recv_buffer[0];
        memcpy(&evt.message.request.payload_len, &recv_buffer[1], 4);

        if (evt.message.request.payload_len > RECV_BUFFER_SIZE)
        {
            LOG_DEBUG("Cannot handle this size of request message\n");
            break;
        }

        // read data
        ret_code = try_read_length(evt.message.request.payload_len);
        if (ret_code != RET_SUCCESS)
        {
            // disconnect
            break;
        }

        if (evt.message.type == MSG_TYPE_RESPONSE)
        {
            evt.message.response.payload_len    = evt.message.request.payload_len - 2;
            evt.message.response.request_type   = recv_buffer[0];
            evt.message.response.ret_code       = recv_buffer[1];
            evt.message.response.p_payload      = recv_buffer + 2;
        }
        else
        {
            evt.message.request.p_payload = recv_buffer;
        }

        evt_handler(&evt);
    }

    evt.type = TCP_EVT_TYPE_DISCONNECTED;
    evt_handler(&evt);

    return NULL;
}


ret_code_t tcp_client_connect(tcp_client_config_t * p_config)
{
    int err;
    pthread_t listener_thread;
    tcp_evt_t tcp_evt;


    m_client_fd = socket(AF_INET, SOCK_STREAM, 0);
    if (m_client_fd == -1)
    {
        perror("Creating socket");
        return RET_ERROR_NO_MEM;
    }

    server.sin_family      = AF_INET;
    server.sin_addr.s_addr = inet_addr(p_config->p_ip_address);
    server.sin_port        = htons(p_config->port);


    err = connect(m_client_fd, (struct sockaddr *)&server, sizeof(server));
    if (err < 0)
    {
        perror("Connecting");
        return RET_ERROR_OTHER;
    }

    tcp_evt.type = TCP_EVT_TYPE_CONNECTED;
    p_config->evt_handler(&tcp_evt);

    err = pthread_create(&listener_thread, NULL, listener_func, p_config->evt_handler);
    if (err != 0)
    {
        perror("Creating pthread");
        return RET_ERROR_OTHER;
    }

    return RET_SUCCESS;
}

ret_code_t tcp_client_disconnect(void)
{
    return RET_SUCCESS;
}

ret_code_t tcp_client_send_data(void * p_data, uint32_t len)
{
    uint32_t sent_len;

    if (m_client_fd == -1)
    {
        return RET_ERROR_INVALID_STATE;
    }

    sent_len = send(m_client_fd, p_data, len, 0);
    if (sent_len == -1)
    {
        perror("Sending data");
        m_client_fd = -1;
        return RET_ERROR_OTHER;
    }
    else if (sent_len != len)
    {
        return RET_ERROR_OTHER;
    }

    return RET_SUCCESS;
}

ret_code_t tcp_client_send_message(message_t * p_message)
{
    ret_code_t ret_code;
    uint8_t    header[5];

    header[0] = p_message->type;

	memcpy(&header[1], &p_message->request.payload_len, sizeof(p_message->request.payload_len));

	ret_code = tcp_client_send_data((void*)header, 5);
	RETURN_ON_ERROR(ret_code);

	// send payload if present
	if (p_message->request.payload_len > 0)
	{
		ret_code = tcp_client_send_data((void*)p_message->request.p_payload,p_message->request.payload_len);
	}


    return ret_code;
}
