#ifndef TCP_CLIENT_H__
#define TCP_CLIENT_H__

#include <stdint.h>
#include "ret_code.h"

#define RECV_BUFFER_SIZE 4000000  /**< The receive buffer size sets the maximum message size of input input message. */


/** @brief Message type defined by the protocol.
 *
 * @details For more details on the interpretation of each message type, see the
 *          specification of the communication protocol.
 *
 * @note    The protocol uses only one byte for coding the message type. The range is therefore limited to 0-255.
 */
typedef enum
{
    MSG_TYPE_GOTO_STATE                 = 0x01,     /**< Message type for changing the current state. This message is sent from host to device. */
    MSG_TYPE_CONFIG_CAMERA              = 0x02,     /**< Message type for configuring camera parameters. This message is sent from host to device. */
    MSG_TYPE_CONFIG_SCAN                = 0x03,     /**< Message type for configuring the scanner. This message is sent from host to device.  */
    MSG_TYPE_GET_IMAGE                  = 0x04,     /**< Message type for obtaining an image. This message is sent from host to device. */
    MSG_TYPE_SCAN_DATA                  = 0x05,     /**< Message type indicating that it contains line data. This message is sent from device to host. */

    MSG_TYPE_RESPONSE                   = 0x80      /**< Message type indication a response to a request. This message is only sent from device to host. */
} message_type_t;


/** @brief Definition of a message.
 *
 * @details For more details on the structure of a message, see the
 *          specification of the communication protocol.
 */
typedef struct
{
    message_type_t  type;                       /**< The type of the message. Only one byte will be used. */

    union {
        struct
        {
            uint8_t     * p_payload;            /**< Pointer to the data associated with the message type. */
            uint32_t    payload_len;            /**< The length with the associated data. */
        } request;                              /**< In case of a request, the message format is defined by this struct. */

        struct
        {
            message_type_t  request_type;       /**< The request type which the response is related to. This is limited to one byte. */
            ret_code_t      ret_code;           /**< The return code to the request. This is limited to one byte. */
            uint8_t         * p_payload;        /**< Pointer to additional data associated with the response. */
            uint32_t        payload_len;        /**< Length of additional data associated with the response. */
        } response;                             /**< In case of a response, that is @ref message_t::type is @ref MSG_TYPE_RESPONSE, the message format is defined by this struct. */
    };

} message_t;


/** @brief Event type sent from the TCP server.*/
typedef enum
{
    TCP_EVT_TYPE_CONNECTED,                     /**< When a host connects, this event type is sent. */
    TCP_EVT_TYPE_DISCONNECTED,                  /**< When a host disconnect, this event type is sent. */
    TCP_EVT_TYPE_RECV_MESSAGE                   /**< When a message is received, this event type is sent. */
} tcp_evt_type_t;


/** @brief Definition of a TCP server event. */
typedef struct
{
    tcp_evt_type_t  type;                       /**< The type of the event. */
    message_t       message;                    /**< If the type is @ref TCP_EVT_TYPE_RECV_MESSAGE, this is the associated message. */
} tcp_evt_t;

/** @brief Definition of a tcp event handler. */
typedef void (*tcp_evt_handler_t)(tcp_evt_t * p_evt);


/** @brief TCP client configuration. */
typedef struct
{
	tcp_evt_handler_t evt_handler;				/**< Event handler which is called every time a TCP event occurs. */
    int               port;						/**< The poer the TCP client should connect to. */
    char              * p_ip_address;			/**< A pointer to the IP address string. */
} tcp_client_config_t;


/** @brief Tries to connect to a client.
 *
 *  @param[in]  p_config        IP Address and port for the connection.
 *
 *  @return RET_SUCCESS         If the server was successfully started.
 *  @return RET_ERROR_NO_MEM    If the application was unable to create a socket.
 *  @return RET_ERROR_OTHER     If some internal socket operation failed, or if failed to connect.
 */
ret_code_t tcp_client_connect(tcp_client_config_t * p_config);


/** @brief Tries to disconnect from the connected client.
 *
 *  @return RET_SUCCESS         If the server was successfully started.
 * 							   else return some error code.
 */
ret_code_t tcp_client_disconnect(void);


/** @brief Sends data to the connected client.
 *
 *  @details This call is blocking.
 *
 *  @param[in] p_data        	Pointer to the raw data to be sent.
 *  @param[in] len				Length of the data in bytes.
 *
 *  @return RET_SUCCESS         	If the data was sent successfully.
 *  @return RET_ERROR_INVALID_STATE If the application was not connected to a client.
 *  @return RET_ERROR_OTHER     	If some internal socket operation failed.
 */
ret_code_t tcp_client_send_data(void * p_data, uint32_t len);


/** @brief Sends a message to the connected client.
 *
 *  @details This call is blocking.
 *
 *  @param[in] p_message      	Pointer to the message to be sent.
 *
 *  @return RET_SUCCESS         	If the data was sent successfully.
 *  @return RET_ERROR_INVALID_STATE If the application was not connected to a client.
 *  @return RET_ERROR_OTHER     	If some internal socket operation failed.
 */
ret_code_t tcp_client_send_message(message_t * p_message);


#endif // TCP_CLIENT_H__
