import matplotlib
import matplotlib.pyplot as plt
import matplotlib.animation as animation
from collections import deque 
import math
import numpy

class LinePlotter(animation.TimedAnimation):
	
	def __init__(self, get_max_height_func,
					get_num_scan_lines_per_image_func,
					get_height_func, 
					get_size_func=None, 
					get_color_func=None):

		self.get_num_scan_lines_per_image = get_num_scan_lines_per_image_func
		self.get_max_height = get_max_height_func
		self.get_height = get_height_func
		self.get_size   = get_size_func
		self.get_color  = get_color_func
		
		fig = plt.figure()
		self.ax1 = fig.add_subplot(111)

		self.ax1.set_xlim([0, get_num_scan_lines_per_image_func()])
		self.ax1.set_ylim([0, get_max_height_func()])

		self.ax1.set_title('Height profile',fontweight="bold", fontsize=20)
		self.ax1.set_xlabel('x')
		self.ax1.set_ylabel('y')

		self.ax1.grid(True, linestyle = '-', color = '0.75')
		self.scat = plt.scatter(0, 0)
				
		animation.TimedAnimation.__init__(self, fig, interval=25)

	def show(self):
		plt.show()

	def new_frame_seq(self):
		return iter(range(1))

	def _draw_frame(self,i):
			
		# Set height
		height = self.get_height()
		if height == None or height == []:
			return
		height = [(h/65536.0) * self.get_max_height() for h in height] 

		self.ax1.set_xlim([0, self.get_num_scan_lines_per_image()])
		self.ax1.set_ylim([0, self.get_max_height()])
		
		coords = zip(range(len(height)), height)
		self.scat.set_offsets(coords)

		# Set sizes
		sizes = None
		if self.get_size != None:
			sizes = self.get_size()
			sizes = [size/256 for size in sizes]
		if sizes == None or sizes == []:
			sizes = [10]*len(height)
		self.scat._sizes = sizes

		# Set grey level
		color = None
		if self.get_color != None:
			color = self.get_color()
			# Group into tuples of (R,G,B)
			color = [ color[i:i+3] for i in range(0, len(color), 3) ]
			color2 = []
			for elem in color:
				color2.append([x/65536.0 for x in elem])
		if color == None or color == []:
			color2 = [(0,0,0)]*len(height)
			

		self.scat.set_color(color2)

class ProcessTimePlotter(animation.TimedAnimation):
	
	def __init__(self, get_processing_time_func, title):
		self.get_processing_time_func = get_processing_time_func
		
		fig = plt.figure()

		fig.suptitle(title,fontweight="bold", fontsize=20)

		formatter = matplotlib.ticker.ScalarFormatter(useOffset=False)

		self.ax1 = fig.add_subplot(121)
		self.ax1.set_ylabel('Time (ms)')
		self.ax1.set_xlabel("Sample #")
		self.ax1.yaxis.set_major_formatter(formatter)
		self.processingTimeData = deque([], 500)
		self.processTimePlot, = self.ax1.plot([],[],linewidth=2)

		self.ax2 = fig.add_subplot(122)
		self.ax2.set_xlabel("Time (ms)")
		self.ax2.set_ylabel("Frequency")
		#self.ax2.xaxis.set_major_formatter(formatter)		
		self.ax2.axes.get_yaxis().set_visible(False)

		animation.TimedAnimation.__init__(self, fig, interval=50)

	def show(self):
		plt.show()

	def new_frame_seq(self):
		return iter(range(1))

	def _draw_frame(self,i):

		processing_time = self.get_processing_time_func()
		if processing_time != None and processing_time != [] and processing_time != 0:
			self.processingTimeData.append(processing_time)

			maxVal = max(self.processingTimeData)
			minVal = min(self.processingTimeData)
			max_min_diff = maxVal - minVal
		
			# plot trend line
			if minVal != maxVal:
				self.ax1.set_ylim(minVal - 0.5*max_min_diff, maxVal + 0.5*max_min_diff)

			x = range(len(self.processingTimeData))
			y = list(self.processingTimeData)
			self.processTimePlot.set_data(x,y)
			self.ax1.set_xlim(0,len(x))

			# plot histogram
			self.ax2.cla()
			binSize = max_min_diff/25
			if binSize == 0:
				binSize = 1
			self.ax2.hist(list(self.processingTimeData), 
									normed=True, 
									facecolor="blue",
									bins=numpy.arange(minVal,maxVal + binSize, binSize))[0]
			if minVal != maxVal:
				self.ax2.set_xlim(minVal,maxVal)

			formatter = matplotlib.ticker.ScalarFormatter(useOffset=False)
			self.ax2.set_xlabel("Time (ms)")
			self.ax2.set_ylabel("Frequency (0-1)")
			self.ax2.xaxis.set_major_formatter(formatter)


class ProcessTimePlotterCombined(animation.TimedAnimation):
	
	def __init__(self, get_processing_time_func, titles):
		self.get_processing_time_func = get_processing_time_func
		self.dim = len(titles);		
		fig = plt.figure()
		fig.suptitle("Processing time combined",fontweight="bold", fontsize=20)

		formatter = matplotlib.ticker.ScalarFormatter(useOffset=False)

		self.lines = []
		self.processingTimeData = []

		self.ax1 = fig.add_subplot(111)
		self.ax1.set_ylabel('Time (ms)')
		self.ax1.set_xlabel("Sample #")
		self.ax1.yaxis.set_major_formatter(formatter)
		for i in range(self.dim):
			lineobj = self.ax1.plot([],[],linewidth=2, label=titles[i])[0]
			self.lines.append(lineobj)		
			self.processingTimeData.append(deque([], 500))
		
		self.ax1.legend()
		
		animation.TimedAnimation.__init__(self, fig, interval=50)

	def show(self):
		plt.show()

	def new_frame_seq(self):
		return iter(range(1))

	def _draw_frame(self,i):

		processing_time = self.get_processing_time_func()
		if None in processing_time:
			return
		if len(processing_time) != self.dim:
			return

		global_min = 10000000000
		global_max = 0

		for i in range(self.dim):
			if (processing_time == None or processing_time == 0):
				break
			self.processingTimeData[i].append(processing_time[i])

			x = range(len(self.processingTimeData[i]))
			y = list(self.processingTimeData[i])
			self.lines[i].set_data(x,y)

			maxVal = max(self.processingTimeData[i])
			global_max = max(maxVal, global_max)		
			minVal = min(self.processingTimeData[i])
			global_min = min(minVal, global_min)
		
			max_min_diff = global_max - global_min
	
			if global_min != global_max:
				self.ax1.set_ylim(global_min - 0.5*max_min_diff, global_max + 0.5*max_min_diff)

			self.ax1.set_xlim(0,len(x))


class ColorPlotter(animation.TimedAnimation):
	
	def __init__(self, get_color_func):
		self.get_color_func = get_color_func
		
		fig = plt.figure()

		fig.suptitle("Color",fontweight="bold", fontsize=20)

		formatter = matplotlib.ticker.ScalarFormatter(useOffset=False)

		self.ax1 = fig.add_subplot(111)
		self.ax1.set_xlabel("x")
		self.ax1.set_ylabel("y")
		self.ax1.set_xlim([0, 1920])
		self.ax1.set_ylim([0,1])
		self.ax1.axes.get_yaxis().set_visible(False)

		animation.TimedAnimation.__init__(self, fig, interval=1000)

	def show(self):
		plt.show()

	def new_frame_seq(self):
		return iter(range(1))

	def _draw_frame(self,i):

		color = self.get_color_func()
		if color != None and color != []:
			x = range(1920)

			# plot histogram
			N, bins, patches = self.ax1.hist(x, 1920, edgecolor="none", facecolor="blue")
			
			for i in range(0, 1920*3, 3):
				patches[i/3].set_facecolor((color[i]/65536.0, color[i+1]/65536.0, color[i+2]/65536.0))

			self.ax1.set_xlabel("x")
			self.ax1.set_ylabel("y")
			self.ax1.set_xlim([0, 1920])
			self.ax1.set_ylim([0,1])


class SavedDataPlotter:
	def __init__(self, filename):
		fig = plt.figure()

		x1 = []
		x2 = []
		currentItem = -1
		with open("../" + filename, 'r') as myfile:
			for line in myfile:
				if line == "\n":
					continue
				if line == "Callback processing time:\n":
					currentItem = 0
				elif line == "Image processing time:\n":	
					currentItem = 1

				elif currentItem == 0:
					x1.append(int(line))
				elif currentItem == 1:
					x2.append(int(line))

		ax1 = fig.add_subplot(121)
		ax1.set_ylabel('Frequency')
		ax1.set_xlabel("Callback processing time (us)")
		ax1.hist(x1)

		ax2 = fig.add_subplot(122)
		ax2.set_ylabel("Frequency")
		ax2.set_xlabel("Image processing time (us)")
		ax2.hist(x2)

		plt.show()

