#include "camera.h"
#include "app_error.h"

#include <string.h>
#include <stdint.h>
#include <stdbool.h>
#include <flycapture/C/FlyCapture2_C.h>
#include <flycapture/C/FlyCapture2Defs_C.h>

#ifdef DEBUG
#define FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code)   \
    do {                                \
        if (err_code != FC2_ERROR_OK)   \
        {                                                                               \
            LOG_DEBUG("FC2 Error occurred on line %d, error: %s\n",__LINE__, fc2ErrorToDescription(err_code)); \
            disconnect();               \
            return RET_ERROR_OTHER;     \
        }                               \
    } while (0)
#else
#define FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code)   \
    do {                                \
        if (err_code != FC2_ERROR_OK)   \
        {                               \
            disconnect();               \
            return RET_ERROR_OTHER;     \
        }                               \
    } while (0)

#endif

static fc2Context m_context;

static bool m_connected;
static uint8_t * p_pinned_buffer;

static fc2Image m_img;
static bool m_capture_started;


static ret_code_t disconnect(void)
{
    fc2Error err_code;

    if (m_connected == false)
    {
        return RET_SUCCESS;
    }

    err_code = fc2Disconnect(m_context);
    if (err_code != FC2_ERROR_OK)
    {
        return RET_ERROR_OTHER;
    }

    err_code = fc2DestroyContext(m_context);
    if (err_code != FC2_ERROR_OK)
    {
        return RET_ERROR_OTHER;
    }

    m_connected = false;

    return RET_SUCCESS;
}


static ret_code_t connect(void)
{
    fc2Error err_code;
    fc2PGRGuid guid;

    if (m_connected == true)
    {
        return RET_SUCCESS;
    }

    err_code = fc2CreateContext(&m_context);
    FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);

    err_code = fc2GetCameraFromIndex(m_context, 0, &guid);
    FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);

    err_code = fc2Connect(m_context, &guid);
    FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);

    m_connected = true;

    return RET_SUCCESS;
}


ret_code_t camera_config(camera_config_t * p_config)
{
    if (p_config == NULL)
    {
        return RET_ERROR_NULL;
    }

    fc2Error err_code;
    ret_code_t ret_code;

    ret_code = connect();
    RETURN_ON_ERROR(ret_code);

    switch (p_config->type)
    {
        case CAMERA_CONFIG_fc2GetConfiguration:
            err_code = fc2GetConfiguration(m_context, &p_config->data.config);
            FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);
            break;

        case CAMERA_CONFIG_fc2SetConfiguration:
            err_code = fc2SetConfiguration(m_context, &p_config->data.config);
            FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);
            break;

        case CAMERA_CONFIG_fc2GetCameraInfo:
            err_code = fc2GetCameraInfo(m_context, &p_config->data.cameraInfo);
            FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);
            break;

        case CAMERA_CONFIG_fc2GetPropertyInfo:
            err_code = fc2GetPropertyInfo(m_context, &p_config->data.propertyInfo);
            FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);
            break;

        case CAMERA_CONFIG_fc2GetProperty:
            err_code = fc2GetProperty(m_context, &p_config->data.property);
            FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);
            break;

        case CAMERA_CONFIG_fc2SetProperty:
            err_code = fc2SetProperty(m_context, &p_config->data.property);
            FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);
            break;

        case CAMERA_CONFIG_fc2GetVideoModeAndFrameRateInfo:
            err_code = fc2GetVideoModeAndFrameRateInfo(m_context,
                                                       p_config->data.video_framerate_info.videoMode,
                                                       p_config->data.video_framerate_info.frameRate,
                                                       &p_config->data.video_framerate_info.supported);
            FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);
            break;

        case CAMERA_CONFIG_fc2GetVideoModeAndFrameRate:
            err_code = fc2GetVideoModeAndFrameRate(m_context,
                                                   &p_config->data.video_framerate_info.videoMode,
                                                   &p_config->data.video_framerate_info.frameRate);
            FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);
            break;

        case CAMERA_CONFIG_fc2SetVideoModeAndFrameRate:
            err_code = fc2SetVideoModeAndFrameRate(m_context,
                                                   p_config->data.video_framerate_info.videoMode,
                                                   p_config->data.video_framerate_info.frameRate);
            FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);
            break;

        case CAMERA_CONFIG_fc2GetFormat7Info:
            err_code = fc2GetFormat7Info(m_context,
                                         &p_config->data.format7Info.format7Info,
                                         &p_config->data.format7Info.supported);
            FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);
            break;

        case CAMERA_CONFIG_fc2ValidateFormat7Settings:
            err_code = fc2ValidateFormat7Settings(m_context,
                                                 &p_config->data.format7Settings.imageSettings,
                                                 &p_config->data.format7Settings.supported,
                                                 &p_config->data.format7Settings.packetInfo);
            FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);
            break;

        case CAMERA_CONFIG_fc2GetFormat7Configuration:
            err_code = fc2GetFormat7Configuration(m_context,
                                                 &p_config->data.format7Configuration.imageSettings,
                                                 &p_config->data.format7Configuration.packetSize,
                                                 &p_config->data.format7Configuration.percentage);
            FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);
            break;

        case CAMERA_CONFIG_fc2SetFormat7ConfigurationPacket:
            err_code = fc2SetFormat7ConfigurationPacket(m_context,
                                                         &p_config->data.format7Configuration.imageSettings,
                                                         p_config->data.format7Configuration.packetSize);
            FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);
            break;

        case CAMERA_CONFIG_fc2SetFormat7Configuration:
            err_code = fc2SetFormat7Configuration(m_context,
                                                 &p_config->data.format7Configuration.imageSettings,
                                                 p_config->data.format7Configuration.percentage);
            FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);

            {
                // In addition Pin host memory
                const double bytesPerPacket = 1024;
                const uint32_t imageSize = // assume to use 2 byte pixels
                        2 * p_config->data.format7Configuration.imageSettings.height
                        * p_config->data.format7Configuration.imageSettings.width;
                const uint32_t numOfPackets = ceil(imageSize/bytesPerPacket);
                const uint32_t bufferSize = numOfPackets * bytesPerPacket;

                fc2Config config;
                err_code = fc2GetConfiguration(m_context, &config);
                FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);

                if (p_pinned_buffer != NULL)
                {
                    cudaFreeHost(p_pinned_buffer);
                }

                cudaError_t cuda_error;
                cuda_error = cudaHostAlloc(&p_pinned_buffer,
                                           bufferSize*config.numBuffers, cudaHostAllocDefault);
                if (cuda_error != cudaSuccess)
                {
                    return RET_ERROR_NO_MEM;
                }

                err_code = fc2SetUserBuffers(m_context, p_pinned_buffer, bufferSize, config.numBuffers);
                FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);
            }
            break;

        case CAMERA_CONFIG_fc2GetEmbeddedImageInfo:
            err_code = fc2GetEmbeddedImageInfo(m_context,
                                               &p_config->data.embeddedImageInfo);
            FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);
            break;

        case CAMERA_CONFIG_fc2SetEmbeddedImageInfo:
            err_code = fc2SetEmbeddedImageInfo(m_context,
                                               &p_config->data.embeddedImageInfo);
            FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);
            break;

        case CAMERA_CONFIG_fc2GetTriggerModeInfo:
            err_code = fc2GetTriggerModeInfo(m_context, &p_config->data.triggerModeInfo);
            FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);
            break;

        case CAMERA_CONFIG_fc2GetTriggerMode:
            err_code = fc2GetTriggerMode(m_context, &p_config->data.triggerMode);
            FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);
            break;

        case CAMERA_CONFIG_fc2SetTriggerMode:
            err_code = fc2SetTriggerMode(m_context, &p_config->data.triggerMode);
            FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);
            break;

        case CAMERA_CONFIG_fc2GetTriggerDelayInfo:
            err_code = fc2GetTriggerDelayInfo(m_context, &p_config->data.triggerDelayInfo);
            FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);
            break;

        case CAMERA_CONFIG_fc2GetTriggerDelay:
            err_code = fc2GetTriggerDelay(m_context, &p_config->data.triggerDelay);
            FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);
            break;

        case CAMERA_CONFIG_fc2SetTriggerDelay:
            err_code = fc2SetTriggerDelay(m_context, &p_config->data.triggerDelay);
            FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);
            break;


        default:

            FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);

    }

    return RET_SUCCESS;
}


ret_code_t camera_get_image(fc2Image * p_img)
{
    if (p_img == NULL)
    {
        return RET_ERROR_NULL;
    }

    ret_code_t ret_code;
    fc2Error   err_code;

    ret_code = connect();
    RETURN_ON_ERROR(ret_code);

    fc2Image temp_image;

    err_code = fc2CreateImage(p_img);
    FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);

    err_code = fc2CreateImage(&temp_image);
    FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);

    err_code = fc2StartCapture(m_context);
    FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);

    err_code = fc2RetrieveBuffer(m_context, &temp_image);
    FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);

    err_code = fc2StopCapture(m_context);
    FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);

    err_code = fc2ConvertImageTo(FC2_PIXEL_FORMAT_RGB16, &temp_image, p_img);
    FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);

    ret_code = disconnect();
    RETURN_ON_ERROR(ret_code);

    return RET_SUCCESS;
}

ret_code_t camera_destroy_image(fc2Image * p_img)
{
    if (p_img == NULL)
    {
        return RET_ERROR_NULL;
    }

    fc2Error err_code;

    err_code = fc2DestroyImage(p_img);
    if (err_code != FC2_ERROR_OK)
    {
        return RET_ERROR_OTHER;
    }

    return RET_SUCCESS;
}

ret_code_t camera_start_capture(void)
{
    ret_code_t ret_code;
    fc2Error   err_code;

    ret_code = connect();
    RETURN_ON_ERROR(ret_code);

    err_code = fc2CreateImage(&m_img);
    FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);

    err_code = fc2StartCapture(m_context);
    FC2_RETURN_AND_DISCONNECT_ON_ERROR(err_code);

    m_capture_started = true;

    return RET_SUCCESS;
}

ret_code_t camera_get_buffer(fc2Image ** pp_img)
{
    if (!m_capture_started)
    {
        return RET_ERROR_INVALID_STATE;
    }

    fc2Error err_code;

    err_code = fc2RetrieveBuffer(m_context, &m_img);
    if (err_code != FC2_ERROR_OK)
    {
        LOG_DEBUG("Failed retrieving buffer\n");
        return RET_ERROR_OTHER;
    }

    *pp_img = &m_img;


    return RET_SUCCESS;
}




ret_code_t camera_stop_capture(void)
{
    if (!m_capture_started)
    {
        return RET_ERROR_INVALID_STATE;
    }

    fc2Error err_code;

    err_code = fc2StopCapture(m_context);
    if (err_code != FC2_ERROR_OK)
    {
        LOG_DEBUG("Failed stopping capture\n");
        return RET_ERROR_OTHER;
    }

    err_code = fc2DestroyImage(&m_img);
    if (err_code != FC2_ERROR_OK)
    {
        return RET_ERROR_OTHER;
    }

    m_capture_started = false;

    return RET_SUCCESS;
}
