#ifndef CAMERA_H__
#define CAMERA_H__

#ifdef __cplusplus
extern "C"
{
#endif

#include <flycapture/C/FlyCapture2Defs_C.h>
#include "ret_code.h"

/** @brief Camera configuration types.
 *
 *  @details The configuration types correspond to the FlyCapture2 SDK configuration calls.
 */
typedef enum
{
    CAMERA_CONFIG_fc2GetConfiguration               = 0x01,
    CAMERA_CONFIG_fc2SetConfiguration               = 0x02,
    CAMERA_CONFIG_fc2GetCameraInfo                  = 0x03,
    CAMERA_CONFIG_fc2GetPropertyInfo                = 0x04,
    CAMERA_CONFIG_fc2GetProperty                    = 0x05,
    CAMERA_CONFIG_fc2SetProperty                    = 0x06,
    CAMERA_CONFIG_fc2GetVideoModeAndFrameRateInfo   = 0x07,
    CAMERA_CONFIG_fc2GetVideoModeAndFrameRate       = 0x08,
    CAMERA_CONFIG_fc2SetVideoModeAndFrameRate       = 0x09,
    CAMERA_CONFIG_fc2GetFormat7Info                 = 0x0A,
    CAMERA_CONFIG_fc2ValidateFormat7Settings        = 0x0B,
    CAMERA_CONFIG_fc2GetFormat7Configuration        = 0x0C,
    CAMERA_CONFIG_fc2SetFormat7ConfigurationPacket  = 0x0D,
    CAMERA_CONFIG_fc2SetFormat7Configuration        = 0x0E,
    CAMERA_CONFIG_fc2GetEmbeddedImageInfo           = 0x0F,
    CAMERA_CONFIG_fc2SetEmbeddedImageInfo           = 0x10,
    CAMERA_CONFIG_fc2GetTriggerModeInfo             = 0x11,
    CAMERA_CONFIG_fc2GetTriggerMode                 = 0x12,
    CAMERA_CONFIG_fc2SetTriggerMode                 = 0x13,
    CAMERA_CONFIG_fc2GetTriggerDelayInfo            = 0x14,
    CAMERA_CONFIG_fc2GetTriggerDelay                = 0x15,
    CAMERA_CONFIG_fc2SetTriggerDelay                = 0x16,

    CAMERA_CONFIG_FORCE32_BIT                       = 0xFFFFFFFF
} camera_config_type_t;


/** @brief Configuration data for the FlyCapture2 SDK calls. */
typedef struct
{
    camera_config_type_t type;                          /**< Type of configuration. */

    union
    {
        fc2Config           config;                         /**< Config struct used when type is @ref CAMERA_CONFIG_fc2GetConfiguration or @ref CAMERA_CONFIG_fc2SetConfiguration. */
        fc2CameraInfo       cameraInfo;                     /**< Config struct used when type is @ref CAMERA_CONFIG_fc2GetCameraInfo. */
        fc2PropertyInfo     propertyInfo;                   /**< Config struct used when type is @ref CAMERA_CONFIG_fc2GetPropertyInfo. */
        fc2Property         property;                       /**< Config struct used when type is @ref CAMERA_CONFIG_fc2GetProperty or @ref CAMERA_CONFIG_fc2SetProperty. */
        fc2TriggerModeInfo  triggerModeInfo;                /**< Config struct used when type is @ref CAMERA_CONFIG_fc2GetTriggerModeInfo. */
        fc2TriggerMode      triggerMode;                    /**< Config struct used when type is @ref CAMERA_CONFIG_fc2GetTriggerMode or @ref CAMERA_CONFIG_fc2SetTriggerMode. */
        fc2TriggerDelayInfo triggerDelayInfo;               /**< Config struct used when type is @ref CAMERA_CONFIG_fc2GetTriggerDelayInfo. */
        fc2TriggerDelay     triggerDelay;                   /**< Config struct used when type is @ref CAMERA_CONFIG_fc2GetTriggerDelay or @ref CAMERA_CONFIG_fc2SetTriggerDelay. */

        struct
        {
            fc2VideoMode videoMode;
            fc2FrameRate frameRate;
            BOOL         supported;
        } video_framerate_info;                         /**< Config struct used when type is @ref CAMERA_CONFIG_fc2GetVideoModeAndFrameRateInfo, @ref CAMERA_CONFIG_fc2GetVideoModeAndFrameRate, or @ref CAMERA_CONFIG_fc2SetVideoModeAndFrameRate. */

        struct
        {
            fc2Format7Info  format7Info;
            BOOL            supported;
        } format7Info;                                  /**< Config struct used when type is @ref CAMERA_CONFIG_fc2GetFormat7Info. */

        struct
        {
            fc2Format7ImageSettings imageSettings;
            fc2Format7PacketInfo    packetInfo;
            BOOL                    supported;
        } format7Settings;                              /**< Config struct used when type is @ref CAMERA_CONFIG_fc2ValidateFormat7Settings.*/

        struct
        {
            fc2Format7ImageSettings imageSettings;
            union
            {
                unsigned int            packetSize;
                float                   percentage;
            };
        } format7Configuration;                         /**< Config struct used when type is @ref CAMERA_CONFIG_fc2GetFormat7Configuration, CAMERA_CONFIG_fc2SetFormat7ConfigurationPacket, or CAMERA_CONFIG_fc2SetFormat7Configuration.*/

        fc2EmbeddedImageInfo    embeddedImageInfo;
    } data;

} camera_config_t;


/** @brief Executes a camera configuration call.
 *
 *  @details If the settings are obtained, set or validated depends on the type of the configuration.
 *
 *  @param[inout]  p_config         The configuration to be executed.
 *
 *  @return RET_SUCCESS             If the configuration was done successfully.
 *  @return RET_ERROR_NULL          If a NULL parameter was provided.
 *  @return RET_ERROR_OTHER         If there was an internal camera configuration error.
 */
ret_code_t camera_config(camera_config_t * p_config);


/** @brief Obtains an image from the camera.
 *
 *  @note The image must be destroyed after it is used using @ref camera_destroy_image. This function is not optimized for speed.
 *
 *  @param[out]  p_img              Pointer to where the image will be stored.
 *
 *  @return RET_SUCCESS             If the image was obtained successfully.
 *  @return RET_ERROR_NULL          If a NULL parameter was provided.
 *  @return RET_ERROR_OTHER         If there was an internal camera configuration error.
 */
ret_code_t camera_get_image(fc2Image * p_img);


/** @brief Destroys an image.
 *
 *  @param[in]  p_img               Pointer to the image to be destroyed.
 *
 *  @return RET_SUCCESS             If the image was destroyed successfully.
 *  @return RET_ERROR_NULL          If a NULL parameter was provided.
 *  @return RET_ERROR_OTHER         If there was an internal camera configuration error.
 */
ret_code_t camera_destroy_image(fc2Image * p_img);


/** @brief Starts capturing images.
 *
 *  @details When an image is captured. When success is returned, it is possible to retrieve a buffer
 *           using the function @ref camera_get_buffer.
 *
 *  @return RET_SUCCESS             If the image was destroyed successfully.
 *  @return RET_ERROR_OTHER         If there was an internal camera configuration error.
 */
ret_code_t camera_start_capture(void);


/** @brief Retrieve an image buffer from the camera
 *
 *  @details When capturing is started, an image can be retrieved using this function
 *
 *  @param[inout] pp_img            Provide a pointer to a pointer where the image should be located. It is updated with the retrieved image.
 *
 *  @return RET_SUCCESS             If an image was successfully retrieved.
 *  @return RET_ERROR_INVALID_STATE If capturing was not started
 */
ret_code_t camera_get_buffer(fc2Image ** pp_img);


/** @brief Stops capturing images.
 *
 * @details. It is now not possible to get a buffer.
 *
 *  @return RET_SUCCESS             If the image was destroyed successfully.
 *  @return RET_ERROR_OTHER         If there was an internal camera configuration error.
 */
ret_code_t camera_stop_capture(void);

#ifdef __cplusplus
}
#endif

#endif
