#include <string.h>
#include <stdio.h>
#include <flycapture/C/FlyCapture2_C.h>
#include <flycapture/C/FlyCapture2Defs_C.h>
#include "config_evt_broker.h"
#include "camera.h"
#include "app_error.h"
#include "log.h"
#include "scanner.h"


static ret_code_t on_camera_config(tcp_evt_t const * p_evt,
                                   message_t * p_response)
{
    ret_code_t ret_code;
    camera_config_t config;

    if (p_evt->message.request.payload_len < 4)
    {
        return RET_ERROR_INVALID_LENGTH;
    }

    memcpy(&config.type, p_evt->message.request.p_payload, 4);

    const uint8_t * p_data   = p_evt->message.request.p_payload + 4;
    const uint32_t  data_len = p_evt->message.request.payload_len - 4;

    switch (config.type)
    {
        case CAMERA_CONFIG_fc2GetConfiguration:         // fall through
        case CAMERA_CONFIG_fc2GetCameraInfo:            // fall through
        case CAMERA_CONFIG_fc2GetVideoModeAndFrameRate: // fall through
        case CAMERA_CONFIG_fc2GetFormat7Configuration:  // fall through
        case CAMERA_CONFIG_fc2GetEmbeddedImageInfo:     // fall through
        case CAMERA_CONFIG_fc2GetTriggerModeInfo:       // fall through
        case CAMERA_CONFIG_fc2GetTriggerMode:           // fall through
        case CAMERA_CONFIG_fc2GetTriggerDelayInfo:      // fall through
        case CAMERA_CONFIG_fc2GetTriggerDelay:
            if (data_len != 0)
            {
                return RET_ERROR_INVALID_LENGTH;
            }

            break;

        case CAMERA_CONFIG_fc2GetPropertyInfo:
            if (data_len != sizeof(config.data.propertyInfo.type))
            {
                return RET_ERROR_INVALID_LENGTH;
            }
            memcpy(&config.data.propertyInfo.type, p_data, data_len);
            break;

        case CAMERA_CONFIG_fc2GetProperty:
            if (data_len != sizeof(config.data.property.type))
            {
                return RET_ERROR_INVALID_LENGTH;
            }
            memcpy(&config.data.property.type, p_data, data_len);
            break;

        case CAMERA_CONFIG_fc2SetConfiguration:
            if (data_len != sizeof(config.data.config))
            {
                return RET_ERROR_INVALID_LENGTH;
            }
            memcpy(&config.data.config, p_data, data_len);
            break;

        case CAMERA_CONFIG_fc2SetProperty:
            if (data_len != sizeof(config.data.property))
            {
                return RET_ERROR_INVALID_LENGTH;
            }
            memcpy(&config.data.property, p_data, data_len);
            break;

        case CAMERA_CONFIG_fc2GetVideoModeAndFrameRateInfo:
        case CAMERA_CONFIG_fc2SetVideoModeAndFrameRate:
            if (data_len != sizeof(config.data.video_framerate_info.frameRate)
                            + sizeof(config.data.video_framerate_info.videoMode))
            {
                return RET_ERROR_INVALID_LENGTH;
            }
            memcpy(&config.data.video_framerate_info, p_data, data_len);
            break;

        case CAMERA_CONFIG_fc2GetFormat7Info:
            if (data_len != sizeof(config.data.format7Info.format7Info.mode))
            {
                return RET_ERROR_INVALID_LENGTH;
            }
            memcpy(&config.data.format7Info.format7Info.mode, p_data, data_len);
            break;

        case CAMERA_CONFIG_fc2ValidateFormat7Settings:
            if (data_len != sizeof(config.data.format7Settings.imageSettings))
            {
                return RET_ERROR_INVALID_LENGTH;
            }
            memcpy(&config.data.format7Settings.imageSettings, p_data, data_len);
            break;

        case CAMERA_CONFIG_fc2SetFormat7ConfigurationPacket:
            if (data_len != sizeof(config.data.format7Configuration.imageSettings)
                        + sizeof(config.data.format7Configuration.packetSize))
            {
                return RET_ERROR_INVALID_LENGTH;
            }
            memcpy(&config.data.format7Configuration.imageSettings, p_data,
                    sizeof(config.data.format7Configuration.imageSettings));
            memcpy(&config.data.format7Configuration.packetSize, p_data + sizeof(config.data.format7Configuration.imageSettings),
                    sizeof(config.data.format7Configuration.packetSize));
            break;

        case CAMERA_CONFIG_fc2SetFormat7Configuration:
            if (data_len != sizeof(config.data.format7Configuration.imageSettings)
                        + sizeof(config.data.format7Configuration.percentage))
            {
                return RET_ERROR_INVALID_LENGTH;
            }
            memcpy(&config.data.format7Configuration.imageSettings, p_data,
                        sizeof(config.data.format7Configuration.imageSettings));
            memcpy(&config.data.format7Configuration.percentage, p_data + sizeof(config.data.format7Configuration.imageSettings),
                        sizeof(config.data.format7Configuration.percentage));
            break;

        case CAMERA_CONFIG_fc2SetEmbeddedImageInfo:
            if (data_len != sizeof(config.data.embeddedImageInfo))
            {
                return RET_ERROR_INVALID_LENGTH;
            }
            memcpy(&config.data.embeddedImageInfo, p_data, sizeof(config.data.embeddedImageInfo));
            break;

        case CAMERA_CONFIG_fc2SetTriggerMode:
            if (data_len != sizeof(config.data.triggerMode))
            {
                return RET_ERROR_INVALID_LENGTH;
            }
            memcpy(&config.data.triggerMode, p_data, sizeof(config.data.triggerMode));
            break;

        case CAMERA_CONFIG_fc2SetTriggerDelay:
            if (data_len != sizeof(config.data.triggerDelay))
            {
                return RET_ERROR_INVALID_LENGTH;
            }
            memcpy(&config.data.triggerDelay, p_data, sizeof(config.data.triggerDelay));
            break;

        default:
            LOG_DEBUG("Received camera config type %d\n", config.type);
            return RET_ERROR_NOT_IMPLEMENTED;

    }

    ret_code = camera_config(&config);

    p_response->response.p_payload =(void*)&config;
    p_response->response.payload_len = 4;

    if (ret_code == RET_SUCCESS)
    {
        switch (config.type)
        {
            case CAMERA_CONFIG_fc2GetConfiguration:
                p_response->response.payload_len += sizeof(config.data.config);
                break;

            case CAMERA_CONFIG_fc2GetCameraInfo:
                p_response->response.payload_len += sizeof(config.data.cameraInfo);
                break;

            case CAMERA_CONFIG_fc2GetVideoModeAndFrameRate:
            case CAMERA_CONFIG_fc2GetVideoModeAndFrameRateInfo:
                p_response->response.payload_len += sizeof(config.data.video_framerate_info);
                break;

            case CAMERA_CONFIG_fc2GetFormat7Configuration:
                p_response->response.payload_len += sizeof(config.data.format7Configuration);
                break;

            case CAMERA_CONFIG_fc2GetPropertyInfo:
                p_response->response.payload_len += sizeof(config.data.propertyInfo);
                break;

            case CAMERA_CONFIG_fc2GetProperty:
                p_response->response.payload_len += sizeof(config.data.property);
                break;

            case CAMERA_CONFIG_fc2GetFormat7Info:
                p_response->response.payload_len += sizeof(config.data.format7Info);
                break;

            case CAMERA_CONFIG_fc2ValidateFormat7Settings:
                p_response->response.payload_len += sizeof(config.data.format7Settings);
                break;

            case CAMERA_CONFIG_fc2GetEmbeddedImageInfo:
                p_response->response.payload_len += sizeof(config.data.embeddedImageInfo);
                break;

            case CAMERA_CONFIG_fc2GetTriggerModeInfo:
                p_response->response.payload_len += sizeof(config.data.triggerModeInfo);
                break;

            case CAMERA_CONFIG_fc2GetTriggerMode:
                p_response->response.payload_len += sizeof(config.data.triggerMode);
                break;

            case CAMERA_CONFIG_fc2GetTriggerDelayInfo:
                p_response->response.payload_len += sizeof(config.data.triggerDelayInfo);
                break;

            case CAMERA_CONFIG_fc2GetTriggerDelay:
                p_response->response.payload_len += sizeof(config.data.triggerDelay);
                break;

            default:
                // no implementation
                break;
        }

    }

    return ret_code;
}


static ret_code_t on_scan_config(tcp_evt_t const * p_evt,
                                 message_t * p_response)
{
    if (p_evt->message.request.payload_len != sizeof(scan_config_t))
    {
        return RET_ERROR_INVALID_LENGTH;
    }

    scan_config_t scan_config;
    memcpy(&scan_config, p_evt->message.request.p_payload, p_evt->message.request.payload_len);

    return scanner_set_config(&scan_config);
}




ret_code_t config_evt_broker_on_tcp_evt(tcp_evt_t const * p_evt,
                                        message_t * p_response)
{
    ret_code_t ret_code;

    switch (p_evt->message.type)
    {
        case MSG_TYPE_CONFIG_CAMERA:
            LOG_DEBUG("Config camera entered\n");
            ret_code = on_camera_config(p_evt, p_response);
            break;

        case MSG_TYPE_CONFIG_SCAN:
            LOG_DEBUG("Config scan entered\n");
            ret_code = on_scan_config(p_evt, p_response);
            break;

        default:
            LOG_DEBUG("Unknown config type\n");
            ret_code = RET_ERROR_NOT_IMPLEMENTED;
            break;
    }

    return ret_code;
}

