#include "frame_buffer.h"
#include <string.h>

#include "log.h"


ret_code_t frame_buffer_init(frame_buffer_t * p_frame_buffer, frame_buffer_init_t * p_init)
{
    if (p_frame_buffer == NULL || p_init == NULL)
    {
        return RET_ERROR_NULL;
    }

    if (p_init->image_height == 0
        || p_init->image_width == 0)
    {
        return RET_ERROR_INVALID_CONFIG;
    }

    cudaError_t cuda_error;

    p_frame_buffer->num_images      = 0;
    p_frame_buffer->max_images      = p_init->max_images;
    p_frame_buffer->p_stream        = p_init->p_stream;
    p_frame_buffer->s_height        = p_init->image_height;
    p_frame_buffer->s_width         = p_init->image_width;

#if defined(FRAME_BUFFER_STORE_DEVICE)
    cuda_error = cudaMalloc((void**)&p_frame_buffer->d_image_buffer,
            p_init->image_width * p_init->image_height * p_init->max_images);
#else
    cuda_error = cudaHostAlloc((void**)&p_frame_buffer->d_image_buffer,
                                p_init->image_width * p_init->image_height * p_init->max_images,
                                cudaHostAllocMapped);
#endif
    if (cuda_error != cudaSuccess)
    {
        p_frame_buffer->d_image_buffer = NULL;
        LOG_DEBUG("No memory for allocating frame buffers: %s\n", cudaGetErrorString(cuda_error));
        return RET_ERROR_NO_MEM;
    }

    p_frame_buffer->image_size      = p_init->image_height * p_init->image_width;

    return RET_SUCCESS;
}


ret_code_t frame_buffer_deinit(frame_buffer_t * p_frame_buffer)
{
    if (p_frame_buffer == NULL)
    {
        return RET_ERROR_NULL;
    }

    if (p_frame_buffer->d_image_buffer != NULL)
    {
        cudaFree(p_frame_buffer->d_image_buffer);
        p_frame_buffer->d_image_buffer = NULL;
    }

    return RET_SUCCESS;
}

ret_code_t frame_buffer_add_image(frame_buffer_t * p_frame_buffer, uint8_t * p_image)
{
    cudaError_t cuda_error = cudaSuccess;
    const uint32_t num_images = p_frame_buffer->num_images;
    const uint32_t image_size = p_frame_buffer->image_size;

    if (num_images < p_frame_buffer->max_images)
    {
#if defined(FRAME_BUFFER_COPY_ASYNC) && defined(FRAME_BUFFER_STORE_DEVICE)
        cuda_error = cudaMemcpyAsync(&p_frame_buffer->d_image_buffer[num_images*image_size],
                                     p_image,
                                     p_frame_buffer->image_size,
                                     cudaMemcpyHostToDevice,
                                     *p_frame_buffer->p_stream);
#elif defined(FRAME_BUFFER_COPY_ASYNC) && defined(FRAME_BUFFER_STORE_HOST)
        cuda_error = cudaMemcpyAsync(&p_frame_buffer->d_image_buffer[num_images*image_size],
                                     p_image,
                                     p_frame_buffer->image_size,
                                     cudaMemcpyHostToHost,
                                     *p_frame_buffer->p_stream);
#elif defined(FRAME_BUFFER_COPY_SYNC) && defined(FRAME_BUFFER_STORE_DEVICE)
        cuda_error = cudaMemcpy(&p_frame_buffer->d_image_buffer[num_images*image_size],
                                p_image,
                                p_frame_buffer->image_size,
                                cudaMemcpyHostToDevice);
#elif defined(FRAME_BUFFER_COPY_SYNC) && defined(FRAME_BUFFER_STORE_HOST)
        memcpy(&p_frame_buffer->d_image_buffer[num_images*image_size],
                        p_image,
                        p_frame_buffer->image_size);
#else
#error "Frame buffer configuration not given"
#endif
        if (cuda_error != cudaSuccess)
        {
            LOG_DEBUG("Failed adding image to buffer: %s\n", cudaGetErrorString(cuda_error));
            return RET_ERROR_OTHER;
        }

        p_frame_buffer->num_images++;
    }
    else
    {
        return RET_ERROR_NO_MEM;
    }

    return RET_SUCCESS;
}

ret_code_t frame_buffer_clear(frame_buffer_t * p_frame_buffer)
{
    if (p_frame_buffer == NULL)
    {
        return RET_ERROR_NULL;
    }

    p_frame_buffer->num_images = 0;

    return RET_SUCCESS;
}

