#ifndef STREAM_DATA_H__
#define STREAM_DATA_H__

#ifdef __cplusplus
extern "C"
{
#endif


#include <driver_types.h>
#include "frame_buffer.h"
#include "scanner.h"


#if !defined(STREAM_DATA_OUTPUT_BUFFER_HOST) && \
    !defined(STREAM_DATA_OUTPUT_BUFFER_DEVICE)
#error "Output buffer configuration not set"
#endif


/** @brief Stream data is a collection of data used for image processing.
 *
 *  @details. Each stream used for image processing holds such a data structure.
 **/
typedef struct
{
    cudaStream_t         stream;                        /**< The CUDA stream corresponding to the stream data. */
    frame_buffer_t       * p_frame_buffers;             /**< Pointers to frame buffers stored in host memory. There is one frame buffer for each image in the scan modes image sequence. */
    frame_buffer_t       * d_frame_buffers;             /**< Similar to p_frame_buffers, but the pointers are located on the device. Therefore, they point to same memory, but the pointers are accesible by the GPU. */
    scan_output_buffer_t * h_scan_output_buffer;        /**< Host pointer to the corresponding output buffer. */
    scan_output_buffer_t * d_scan_output_buffer;        /**< Device pointer to the corresponding output buffer. The points to the same memory as h_scan_output_buffer. */
    uint8_t              * d_scan_output_buffer_raw;    /**< Pointer to raw output buffer. */

    cudaEvent_t          start_event;                   /**< CUDA event triggered when image processing starts. */
    cudaEvent_t          gpu_finished_event;            /**< CUDA event triggered when the image processing kernel is finished. */
    cudaEvent_t          sending_finished_event;        /**< CUDA event triggered when TCP send is finished. */
    float                prev_gpu_processing_time;      /**< Stores the previous GPU processing time. The value is stored in milliseconds. */
    float                prev_total_processing_time;    /**< Previous total processing time. The value is stored in milliseconds. */
} stream_data_t;


/** @brief Initializes a stream data structure.
 *
 *  @details    Memory is allocated for all elements in the stream data structure. On failure, all data is deinitialized.
 *
 *  @param[inout]  p_stream_data    Pointer to the stream data structure.
 *  @param[in]     seq_length       The length of the used image sequence. This parameter determines the number of frame buffers.
 *  @param[in]     p_config         The used scan configuration.
 *
 *  @returns    RET_SUCCESS         On successful initialization.
 *  @return     RET_ERROR_NO_MEM    If memory allocation fails.
 */
ret_code_t stream_data_init(stream_data_t * p_stream_data,
                            uint8_t seq_length,
                            scan_config_t * p_config);


/** @brief Deinitializes a stream data structure and its corresponding memory.
 *
 *  @param[inout]  p_stream_data    Pointer to the stream data structure.
 *  @param[in]     seq_length       The length of the used image sequence. This parameter determines the number of frame buffers.
 *
 *  @returns    RET_SUCCESS         On successful deinitialization.
 */
void       stream_data_deinit(stream_data_t * p_stream_data, uint8_t seq_length);

#ifdef __cplusplus
}
#endif

#endif // STREAM_DATA_H__
