#ifndef SCAN_CONFIG_H__
#define SCAN_CONFIG_H__

#include <stdint.h>
#include "ret_code.h"


/** @brief This enum defines the GPIO pin values for the Jetson TK1 and TX1. */
typedef enum
{
	// Jetson TK1 pins
	gpio57 = 57,    // J3A1 - Pin 50
	gpio160 = 160,    // J3A2 - Pin 40
	gpio161 = 161,    // J3A2 - Pin 43
	gpio162 = 162,    // J3A2 - Pin 46
	gpio163 = 163,    // J3A2 - Pin 49
	gpio164 = 164,    // J3A2 - Pin 52
	gpio165 = 165,    // J3A2 - Pin 55
	gpio166 = 166,    // J3A2 - Pin 58

	// Jetson TX1 pins
	gpio36 = 36,      // J21 - Pin 32 - Unused - AO_DMIC_IN_CLK
	gpio37 = 37,      // J21 - Pin 16 - Unused - AO_DMIC_IN_DAT
	gpio38 = 38,      // J21 - Pin 13 - Bidir  - GPIO20/AUD_INT
	gpio63 = 63,      // J21 - Pin 33 - Bidir  - GPIO11_AP_WAKE_BT
	gpio184 = 184,    // J21 - Pin 18 - Input  - GPIO16_MDM_WAKE_AP
	gpio186 = 186,    // J21 - Pin 31 - Input  - GPIO9_MOTION_INT
	gpio187 = 187,    // J21 - Pin 37 - Output - GPIO8_ALS_PROX_INT
	gpio219 = 219     // J21 - Pin 29 - Output - GPIO19_AUD_RST
} gpio_pin_t;


/**@brief The defined scan modes.
*
* @details Up to 2^32 scan modes can be defined. In the description the frame types are abbreviated as following:
*
* L: An image containing the laser line.
* B: An image where the laser line and LED is turned off.
* C: An illuminated by LEDs.
*/
typedef enum
{
	SCAN_MODE_0 = 0,    /**< L, L, L, ... */
	SCAN_MODE_1 = 1,    /**< L, C, L, C, ... */
	SCAN_MODE_2 = 2,    /**< C, C, C, ... */
	SCAN_MODE_3 = 3,    /**< L, B, L, B, ... */
	SCAN_MODE_4 = 4,    /**< L, B, C, L, B, C, ... */

	SCAN_MODE_FORCE_32_BIT = 0xFFFFFFFF
} scan_mode_t;

/**@brief The image info structure holds information about image configuration. */
typedef struct image_info_s
{
	uint32_t            frame_height;               /**< The height of the image in pixels. */
	uint32_t            num_scan_lines_per_image;   /**< The number of scan lines per image. That is the number of columns divided by two. */
} image_info_t;


/** @brief Image processing configuration. */
typedef struct
{
	uint32_t scatter_distance;      /**< Scatter distance is the intensity of the pixel calculated some distance from the height. Defined in pixels. */
	uint32_t threshold_value;       /**< The threshold value sets a lower limit of pixel intensities. All values below are not used for height and reflectance calculations. */
	float    pixel_power;           /**< The exponent used for height calculation. */

	uint32_t num_streams;           /**< The number of CUDA streams used for image processing. */
	uint32_t threads_per_block;     /**< The threads per block is a kernel launch parameter. */
	uint32_t frames_per_message;    /**< Frames per message indicates the size of the frame buffer and output message. */
} img_processing_config_t;


/** @brief Scanner configuration. */
typedef struct scan_config_s
{
	scan_mode_t              scan_mode;                     /**< The active scan mode used to retrieve the image parameters. */
	image_info_t             image_info;                    /**< The image size used for parameter extraction. */
	gpio_pin_t               scanner_ready_pin;             /**< The GPIO indicating when the scanner framework is busy. */
	float                    fps;                           /**< The expected camera frame rate. This is used to see if frames are dropped. */
	float                    cycle_time_diff_threshold_ms;  /**< A threshold value determining the maximum jitter in image reception time.*/

	img_processing_config_t  img_processing_config;         /**< The image processing configuration used for parameter extraction. */
} scan_config_t;


/** @brief Sends all scanner configuration parameters.
*
*	@details All parameters are set in the implementation file.
*
*  @return RET_SUCCESS		If configuration was successful
*  else return some error code.
*/
ret_code_t scan_config_send_all(void);


#endif // OUTPUT_FORMAT_CONFIG_H__