#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <winsock2.h>
#include <ws2tcpip.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdbool.h>


// Need to link with Ws2_32.lib, Mswsock.lib, and Advapi32.lib
#pragma comment (lib, "Ws2_32.lib")
#pragma comment (lib, "Mswsock.lib")
#pragma comment (lib, "AdvApi32.lib")

#include <string.h>


#include "log.h"
#include "tcp_client.h"
#include "app_error.h"

static uint8_t recv_buffer[RECV_BUFFER_SIZE];

#define HEADER_SIZE         5

static struct sockaddr_in server;

static HANDLE threadHandle;
static DWORD  threadId;
static volatile bool   m_stay_connected;

SOCKET m_client_socket = INVALID_SOCKET;

static ret_code_t try_read_length(uint32_t length)
{
	uint32_t bytes_read, temp_bytes_read;

	bytes_read = 0;
	while (bytes_read < length)
	{
		temp_bytes_read = recv(m_client_socket, &recv_buffer[bytes_read], length - bytes_read, 0);
		if (temp_bytes_read <= 0)
		{
			return RET_ERROR_OTHER;
		}

		bytes_read += temp_bytes_read;

	}
	return RET_SUCCESS;
}


DWORD WINAPI listener_func( LPVOID args)
{
	ret_code_t          ret_code;
	tcp_evt_t           evt;
	tcp_evt_handler_t   evt_handler;

	evt_handler = (tcp_evt_handler_t)args;

	LOG_DEBUG("TCP listener thread started\n");

	while (m_stay_connected)
	{
		// read header
		ret_code = try_read_length(HEADER_SIZE);
		if (ret_code != RET_SUCCESS)
		{
			// disconnect
			break;
		}

		evt.type = TCP_EVT_TYPE_RECV_MESSAGE;

		// store header information in the event
		evt.message.type = (message_type_t)recv_buffer[0];
		memcpy(&evt.message.request.payload_len, &recv_buffer[1], 4);

		if (evt.message.request.payload_len > RECV_BUFFER_SIZE)
		{
			LOG_DEBUG("Cannot handle this size of request message\n");
			break;
		}

		// read data
		ret_code = try_read_length(evt.message.request.payload_len);
		if (ret_code != RET_SUCCESS)
		{
			// disconnect
			break;
		}

		if (evt.message.type == MSG_TYPE_RESPONSE)
		{
			evt.message.response.payload_len = evt.message.request.payload_len - 2;
			evt.message.response.request_type = recv_buffer[0];
			evt.message.response.ret_code = recv_buffer[1];
			evt.message.response.p_payload = recv_buffer + 2;
		}
		else
		{
			evt.message.request.p_payload = recv_buffer;
		}

		evt_handler(&evt);
	}

	evt.type = TCP_EVT_TYPE_DISCONNECTED;
	evt_handler(&evt);

	return 0;
}


ret_code_t tcp_client_connect(tcp_client_config_t * p_config)
{
	int err;
	
	tcp_evt_t tcp_evt;
	WSADATA wsaData;

	// Initialize Winsock
	err = WSAStartup(MAKEWORD(2, 2), &wsaData);
	if (err != 0) {
		LOG("WSAStartup failed with error: %d\n", err);
		return RET_ERROR_OTHER;
	}

	m_client_socket = socket(AF_INET, SOCK_STREAM, 0);
	if (m_client_socket == INVALID_SOCKET)
	{
		LOG("Failed creating socket\n");
		WSACleanup();
		return RET_ERROR_OTHER;
	}

	server.sin_family = AF_INET;
	server.sin_addr.s_addr = inet_addr(p_config->p_ip_address);
	server.sin_port = htons(p_config->port);


	err = connect(m_client_socket, (struct sockaddr *)&server, sizeof(server));
	if (err < 0)
	{
		perror("Connecting");
		return RET_ERROR_OTHER;
	}

	tcp_evt.type = TCP_EVT_TYPE_CONNECTED;
	p_config->evt_handler(&tcp_evt);

	m_stay_connected = true;

	threadHandle = CreateThread(NULL, 0, listener_func, p_config->evt_handler, 0, &threadId);
	if (threadHandle == NULL)
	{
		LOG("Failed creating TCP thread\n");
		return RET_ERROR_OTHER;
	}

	return RET_SUCCESS;
}

ret_code_t tcp_client_disconnect()
{
	m_stay_connected = false;

	TerminateThread(threadHandle, 0);

	WaitForSingleObject(threadHandle, INFINITE);

	return RET_SUCCESS;
}

ret_code_t tcp_client_send_data(void * p_data, uint32_t len)
{
	uint32_t sent_len;

	if (m_client_socket == -1)
	{
		return RET_ERROR_INVALID_STATE;
	}

	sent_len = send(m_client_socket, p_data, len, 0);
	if (sent_len == -1)
	{
		perror("Sending data");
		m_client_socket = INVALID_SOCKET;
		return RET_ERROR_OTHER;
	}
	else if (sent_len != len)
	{
		return RET_ERROR_OTHER;
	}

	return RET_SUCCESS;
}

ret_code_t tcp_client_send_message(message_t * p_message)
{
	ret_code_t ret_code;
	uint8_t    header[5];

	header[0] = p_message->type;

	memcpy(&header[1], &p_message->request.payload_len, sizeof(p_message->request.payload_len));

	ret_code = tcp_client_send_data((void*)header, 5);
	RETURN_ON_ERROR(ret_code);

	// send payload if present
	if (p_message->request.payload_len > 0)
	{
		ret_code = tcp_client_send_data((void*)p_message->request.p_payload, p_message->request.payload_len);
	}


	return ret_code;
}
